@@ -128,3 +128,16 @@ class VolumeGroupNotFound(BrickException):
 
 class VolumeGroupCreationFailed(BrickException):
     message = _("Failed to create Volume Group: %(vg_name)s")
+
+
+class CommandExecutionFailed(BrickException):
+    message = _("Failed to execute command %(cmd)s")
+
+
+class VolumeDriverException(BrickException):
+    message = _('An error occurred while IO to volume %(name)s.')
+
+
+class InvalidIOHandleObject(BrickException):
+    message = _('IO handle of %(protocol)s has wrong object '
+                'type %(actual_type)s.')
