@@ -18,6 +18,7 @@
 import hashlib
 import os
 import re
+import tempfile
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
@@ -34,34 +35,39 @@ class RemoteFsClient(object):
     def __init__(self, mount_type, root_helper,
                  execute=putils.execute, *args, **kwargs):
 
-        self._mount_type = mount_type
-        if mount_type == "nfs":
-            self._mount_base = kwargs.get('nfs_mount_point_base', None)
-            if not self._mount_base:
-                raise exception.InvalidParameterValue(
-                    err=_('nfs_mount_point_base required'))
-            self._mount_options = kwargs.get('nfs_mount_options', None)
-            self._check_nfs_options()
-        elif mount_type == "cifs":
-            self._mount_base = kwargs.get('smbfs_mount_point_base', None)
-            if not self._mount_base:
-                raise exception.InvalidParameterValue(
-                    err=_('smbfs_mount_point_base required'))
-            self._mount_options = kwargs.get('smbfs_mount_options', None)
-        elif mount_type == "glusterfs":
-            self._mount_base = kwargs.get('glusterfs_mount_point_base', None)
-            if not self._mount_base:
-                raise exception.InvalidParameterValue(
-                    err=_('glusterfs_mount_point_base required'))
-            self._mount_options = None
-        else:
+        mount_type_to_option_prefix = {
+            'nfs': 'nfs',
+            'cifs': 'smbfs',
+            'glusterfs': 'glusterfs',
+            'vzstorage': 'vzstorage',
+            'quobyte': 'quobyte'
+        }
+
+        if mount_type not in mount_type_to_option_prefix:
             raise exception.ProtocolNotSupported(protocol=mount_type)
+
+        self._mount_type = mount_type
+        option_prefix = mount_type_to_option_prefix[mount_type]
+
+        self._mount_base = kwargs.get(option_prefix + '_mount_point_base')
+        if not self._mount_base:
+            raise exception.InvalidParameterValue(
+                err=_('%s_mount_point_base required') % option_prefix)
+
+        self._mount_options = kwargs.get(option_prefix + '_mount_options')
+
+        if mount_type == "nfs":
+            self._check_nfs_options()
+
         self.root_helper = root_helper
         self.set_execute(execute)
 
     def set_execute(self, execute):
         self._execute = execute
 
+    def get_mount_base(self):
+        return self._mount_base
+
     def _get_hash_str(self, base_str):
         """Return a string that represents hash of base_str
         (in a hex format).
@@ -101,6 +107,8 @@ class RemoteFsClient(object):
         self._execute('mkdir', '-p', mount_path, check_exit_code=0)
         if self._mount_type == 'nfs':
             self._mount_nfs(share, mount_path, flags)
+        elif self._mount_type == 'vzstorage':
+            self._mount_vzstorage(share, mount_path, flags)
         else:
             self._do_mount(self._mount_type, share, mount_path,
                            self._mount_options, flags)
@@ -140,6 +148,53 @@ class RemoteFsClient(object):
                                        % {'sh': nfs_share,
                                           'error': mnt_errors})
 
+    def _vzstorage_write_mds_list(self, cluster_name, mdss):
+        tmp_dir = tempfile.mkdtemp(prefix='vzstorage-')
+        tmp_bs_path = os.path.join(tmp_dir, 'bs_list')
+        with open(tmp_bs_path, 'w') as f:
+            for mds in mdss:
+                f.write(mds + "\n")
+
+        conf_dir = os.path.join('/etc/pstorage/clusters', cluster_name)
+        if os.path.exists(conf_dir):
+            bs_path = os.path.join(conf_dir, 'bs_list')
+            self._execute('cp', '-f', tmp_bs_path, bs_path,
+                          root_helper=self.root_helper, run_as_root=True)
+        else:
+            self._execute('cp', '-rf', tmp_dir, conf_dir,
+                          root_helper=self.root_helper, run_as_root=True)
+        self._execute('chown', '-R', 'root:root', conf_dir,
+                      root_helper=self.root_helper, run_as_root=True)
+
+    def _mount_vzstorage(self, vz_share, mount_path, flags=None):
+        m = re.search("(?:(\S+):\/)?([a-zA-Z0-9_-]+)(?::(\S+))?", vz_share)
+        if not m:
+            msg = (_("Invalid Virtuozzo Storage share specification: %r."
+                     "Must be: [MDS1[,MDS2],...:/]<CLUSTER NAME>[:PASSWORD].")
+                   % vz_share)
+            raise exception.BrickException(msg)
+
+        mdss = m.group(1)
+        cluster_name = m.group(2)
+        passwd = m.group(3)
+
+        if mdss:
+            mdss = mdss.split(',')
+            self._vzstorage_write_mds_list(cluster_name, mdss)
+
+        if passwd:
+            self._execute('pstorage', '-c', cluster_name, 'auth-node', '-P',
+                          process_input=passwd,
+                          root_helper=self.root_helper, run_as_root=True)
+
+        mnt_cmd = ['pstorage-mount', '-c', cluster_name]
+        if flags:
+            mnt_cmd.extend(flags)
+        mnt_cmd.extend([mount_path])
+
+        self._execute(*mnt_cmd, root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=0)
+
     def _check_nfs_options(self):
         """Checks and prepares nfs mount type options."""
         self._nfs_mount_type_opts = {'nfs': self._mount_options}
@@ -174,3 +229,36 @@ class RemoteFsClient(object):
         opt = '%s=%s' % (option, value) if value else option
         opts.append(opt)
         return ",".join(opts) if len(opts) > 1 else opts[0]
+
+
+class ScalityRemoteFsClient(RemoteFsClient):
+    def __init__(self, mount_type, root_helper,
+                 execute=None, *args, **kwargs):
+        self._mount_type = mount_type
+        self._mount_base = kwargs.get(
+            'scality_mount_point_base', "").rstrip('/')
+        if not self._mount_base:
+            raise exception.InvalidParameterValue(
+                err=_('scality_mount_point_base required'))
+        self.root_helper = root_helper
+        self.set_execute(execute or putils.execute)
+        self._mount_options = None
+
+    def get_mount_point(self, device_name):
+        return os.path.join(self._mount_base,
+                            device_name,
+                            "00")
+
+    def mount(self, share, flags=None):
+        """Mount the Scality ScaleOut FS.
+
+        The `share` argument is ignored because you can't mount several
+        SOFS at the same type on a single server. But we want to keep the
+        same method signature for class inheritance purpose.
+        """
+        if self._mount_base in self._read_mounts():
+            LOG.info(_LI('Already mounted: %s'), self._mount_base)
+            return
+        self._execute('mkdir', '-p', self._mount_base, check_exit_code=0)
+        super(ScalityRemoteFsClient, self)._do_mount(
+            'sofs', '/etc/sfused.conf', self._mount_base)
